@@ -5,7 +5,7 @@
  * session.c
  *
  * Copyright (c) 2008 Travis Watkins <amaranth@ubuntu.com>
- * Copyright (c) 2008 Danny Baumann <maniac@opencompositing.org>
+ * Copyright (c) 2008 Danny Baumann <dannybaumann@web.de>
  * Copyright (c) 2006 Patrick Niklaus
  *
  *
@@ -843,6 +843,9 @@ sessionHandleEvent (CompDisplay *d,
 
     SESSION_DISPLAY (d);
 
+    w = NULL;
+    state = 0;
+
     switch (event->type) {
     case MapRequest:
 	w = findWindowAtDisplay (d, event->xmaprequest.window);
@@ -934,26 +937,6 @@ sessionSessionEvent (CompCore         *c,
     WRAP (sc, c, sessionEvent, sessionSessionEvent);
 }
 
-static void
-sessionObjectAdd (CompObject *parent,
-		  CompObject *object)
-{
-    static ObjectAddProc dispTab[] = {
-	(ObjectAddProc) 0, /* CoreAdd */
-	(ObjectAddProc) 0, /* DisplayAdd */
-	(ObjectAddProc) 0, /* ScreenAdd */
-	(ObjectAddProc) sessionWindowAdd
-    };
-
-    SESSION_CORE (&core);
-
-    UNWRAP (sc, &core, objectAdd);
-    (*core.objectAdd) (parent, object);
-    WRAP (sc, &core, objectAdd, sessionObjectAdd);
-	
-    DISPATCH (object, dispTab, ARRAY_SIZE (dispTab), (parent, object));
-}
-
 static Bool
 sessionWindowAddTimeout (void *closure)
 {
